? around-eval-for-emacs-hook.diff
Index: swank.lisp
===================================================================
RCS file: /project/slime/cvsroot/slime/swank.lisp,v
retrieving revision 1.803
diff -U 8 -r1.803 swank.lisp
--- swank.lisp	11 Jan 2013 09:00:30 -0000	1.803
+++ swank.lisp	15 Jan 2013 16:33:44 -0000
@@ -175,16 +175,62 @@
 
 (defvar *pre-reply-hook* '()
   "Hook run (without arguments) immediately before replying to an RPC.")
 
 (defvar *after-init-hook* '()
   "Hook run after user init files are loaded.")
 
 
+;;;; Around Hooks
+;;;
+;;; Around hook is a function with at lease one argument NEXT-FN
+;;; and possible other arguments.
+;;; If the hook calls (FUNCALL NEXT-FN OTHER ARGUMENTS) it invokes
+;;; invokes either the next hook in the hook chain,
+;;; or the original body executed within the around hooks
+;;; in case the current hook is the last one.
+
+(defun call-within-hooks (around-hooks fn &rest arguments)
+  (if around-hooks
+      (apply (car around-hooks)
+             (lambda ()
+               (call-within-hooks (cdr around-hooks) fn))
+             arguments)
+      (apply fn arguments)))
+
+(defmacro within-hooks ((around-hooks) &body body)
+  `(call-within-hooks ,around-hooks (lambda () ,@body)))
+
+#| Example of how around hooks work
+
+(flet ((angle-wrap (next-fn)
+         (princ "<") (funcall next-fn) (princ ">"))
+       (square-wrap (next-fn)
+         (princ "[") (funcall next-fn) (princ "]")))
+
+  (within-hooks ((list #'angle-wrap))
+    (princ "hello"))
+  ;; printed <hello>
+
+  (within-hooks ((list #'angle-wrap #'square-wrap))
+    (princ "hello"))
+  ;; printed <[hello]>
+
+  (within-hooks ((list #'square-wrap #'angle-wrap))
+    (princ "hello"))
+  ;; printed [<hello>]
+  )
+
+|#
+
+(defvar *around-eval-for-emacs-hook* '()
+  "Hook run (with empty argument list) by EVAL-FOR-EMACS around the expression received with an RPC.")
+
+
 ;;;; Connections
 ;;;
 ;;; Connection structures represent the network connections between
 ;;; Emacs and Lisp. Each has a socket stream, a set of user I/O
 ;;; streams that redirect to Emacs, and optionally a second socket
 ;;; used solely to pipe user-output to Emacs (an optimization).  This
 ;;; is also the place where we keep everything that needs to be
 ;;; freed/closed/killed when we disconnect.
@@ -1714,17 +1760,18 @@
          (let ((*buffer-package* (guess-buffer-package buffer-package))
                (*buffer-readtable* (guess-buffer-readtable buffer-package))
                (*pending-continuations* (cons id *pending-continuations*)))
            (check-type *buffer-package* package)
            (check-type *buffer-readtable* readtable)
            ;; APPLY would be cleaner than EVAL. 
            ;; (setq result (apply (car form) (cdr form)))
            (handler-bind ((t (lambda (c) (setf condition c))))
-             (setq result (with-slime-interrupts (eval form))))
+             (setq result (with-slime-interrupts (within-hooks (*around-eval-for-emacs-hook*)
+                                                   (eval form)))))
            (run-hook *pre-reply-hook*)
            (setq ok t))
       (send-to-emacs `(:return ,(current-thread)
                                ,(if ok
                                     `(:ok ,result)
                                     `(:abort ,(prin1-to-string condition)))
                                ,id)))))
 
